##### ctsmod software, copyright 2012 Coryn A.L. Bailer-Jones
##### http://www.mpia.de/homes/calj
##### This software is released under the Creative Commons 
##### Attribution-NonCommercial-ShareAlike 3.0 Unported (CC BY-NC-SA 3.0) license.
##### See the file "license" for more details

# This file: setup_ctsmod.R v23
# Set up parameters for a ctsmod run. Sourced by run_ctsmod.R

##### Lists of permitted things (from ctsmod_models.R; do not edit this list)

obstypes    <- c('fromFile', 'simAsFile', 'simConst', 'simLinear', 'simSinusoid', 'simQuasiSinusoid', 'simAsFuncQuasiSinusoid', 'simOUprocess', 'simIntOUprocess', 'simWienerprocess')
shrinktypes <- c('none', 'only_t', 'only_z', 'both')
priortypes  <- c('Gamma', 'Gaussian')

##### Set up run parameters. Choices of models and their theta parameters are listed in ctsmod_models.R

outStem <- 'test'
warnToFile <- FALSE # send warnings to an output file (name <outStem>.warnings)?
sendEmail  <- FALSE # send an email to EmailAddress on completion?
EmailAddress <- 'xxx@yyy.zzz'
#
obssel  <- as.character(factor('simLinear', obstypes))
#obsdataFname <- '../data/calar3.lcurve'
obsdataHeader <- TRUE # is there a single line header? Default is TRUE
simseed <- 100 # only for generating simulated data
# tzlim are the widest limits on the t,z used in the integral in the event likelihood calculation.
# Must be set if doing 1D or 2D integration. May be shrunk further according to shrink.
# Ignored if doing 0D integration (could set to NA).
tzlim  <- NA # c(-50, 150, -100, 100) # c(t_lower, t_upper, z_lower, z_upper)
shrink <- as.character(factor('none', shrinktypes)) # Don't shrink limits on z if not bypassing TSMod2
negtsd <- TRUE # time uncertainties are negligible (and therefore want to force 0D integration - see header)
#
MeasMod <- new("TimeUniformSignalGaussian")
TSMod1  <- new("FuncLinear", priorform="Gaussian")
TSMod2  <- new("ProbGaussian")
TSMod3  <- new("ProbUniform")
# thetaXflag: TRUE=adjust; FALSE=hold fixed (I use logical vectors rather than lists because I use them for indexing)
# A check is made in the checks section that the names are complete and agree with the prototypes of TSModX@thetaX
theta1Flag <- c(slope=TRUE, tmid=TRUE, offset=TRUE)
theta2Flag <- c(sd=TRUE)
theta3Flag <- c(tmin=FALSE, tmax=FALSE)
# Set parameters of prior PDFs over theta [alpha parameters]
# A check is made of list names with prototype, but no check is made of the values.
# If we fail to set alpha for adjustable parameters, this is trapped later (see checks section).
# If alpha is set for a fixed parameter then this will be used to calculate its prior. Otherwise the prior is 1.
# For a gamma distribution: scale=var/mean, shape=mean^2/var
# Examples:
cat("Checking and assigning alpha (prior parameter) values\n")
TSMod1@alpha1$offset    <- assign.full(TSMod1@alpha1$offset,   list(mean=0, sd=25))      # FuncUniform/FuncLinear,  normal
#TSMod1@alpha1$offset    <- assign.full(TSMod1@alpha1$offset,   list(scale=1, shape=2))   # FuncUniform/FuncLinear,  gamma
#TSMod1@alpha1$sigamp    <- assign.full(TSMod1@alpha1$sigamp,   list(scale=1, shape=2))   # FuncSigmoid,  gamma
#TSMod1@alpha1$lambda    <- assign.full(TSMod1@alpha1$lambda,   list(mean=0,  sd=1))      # FuncSigmoid,  normal
#TSMod1@alpha1$tzero     <- assign.full(TSMod1@alpha1$tzero,    list(mean=0,  sd=1))      # FuncSigmoid,  normal
#TSMod1@alpha1$sinamp    <- assign.full(TSMod1@alpha1$sinamp,   list(scale=1, shape=2))   # FuncSinusoid/FuncQuasiSinusoid, gamma
#TSMod1@alpha1$freq      <- assign.full(TSMod1@alpha1$freq,     list(scale=1, shape=2))   # FuncSinusoid/FuncQuasiSinusoid, gamma
#TSMod1@alpha1$vphaseMAD <- assign.full(TSMod1@alpha1$vphasesd, list(scale=0.1, shape=2)) # FuncQuasiSinusoid, gamma
TSMod1@alpha1$slope  <- assign.full(TSMod1@alpha1$slope,       list(sign="either"))     # FuncLinear,   discrete
TSMod1@alpha1$tmid   <- assign.full(TSMod1@alpha1$tmid,        list(mean=50,  sd=25))   # FuncLinear,   normal
TSMod2@alpha2$sd     <- assign.full(TSMod2@alpha2$sd,          list(scale=5, shape=1.5))  # ProbGaussian, gamma
#TSMod2@alpha2$relax      <- assign.full(TSMod2@alpha2$relax,      list(scale=1, shape=2))  # ProbOUprocess, gamma
#TSMod2@alpha2$diffcon    <- assign.full(TSMod2@alpha2$diffcon,    list(scale=1, shape=2))  # ProbOUprocess/ProbWienerprocess, gamma
#TSMod2@alpha2$offset     <- assign.full(TSMod2@alpha2$offset,     list(mean=0,  sd=1))     # ProbOUprocess/ProbWienerprocess, normal
#TSMod2@alpha2$zstartmean <- assign.full(TSMod2@alpha2$zstartmean, list(mean=0,  sd=1))     # ProbOUprocess/ProbWienerprocess, normal
#TSMod2@alpha2$zstartsd   <- assign.full(TSMod2@alpha2$zstartsd,   list(scale=0, shape=2))  # ProbOUprocess/ProbWienerprocess, gamma
#TSMod2@alpha2$relax   <- assign.full(TSMod2@alpha2$relax,   list(scale=1, shape=2))  # ProbOUprocNaive, gamma
#TSMod2@alpha2$diffcon <- assign.full(TSMod2@alpha2$diffcon, list(scale=1, shape=2))  # ProbOUprocNaive, gamma
#TSMod2@alpha2$xzero   <- assign.full(TSMod2@alpha2$xzero,   list(mean=0,  sd=1))     # ProbOUprocNaive, gamma
#TSMod2@alpha2$wzero   <- assign.full(TSMod2@alpha2$wzero,   list(mean=0,  sd=1))     # ProbIntOUprocNaive, gamma
# Note: Need to do "TSMod2@TSMod1SC <- TSMod1" before using TSMod2, but this is done "just in time" as needed
#
# Set fixed parameter values (i.e. for those not to be sampled over) as a named list.
# A check is made in the check section to ensure that all parameters set as fixed in thetaXFlag are assigned here,
# but the validity of those assignments is not checked. We must assign numeric values here (e.g. no NAs).
# Note that thetaXFlag takes priority over whether a parameter is to be sampled, i.e. fixed values of parameters
# which are to be adjusted will just be ignored.
# If nothing is fixed, then the thetaXFixed is never used, so we can omit, i.e. set to list() or NULL
theta1Fixed <- NULL
theta2Fixed <- NULL
theta3Fixed <- list(tmin=0, tmax=100)

nomodPar  <- theta3Fixed   # list(tmin, tmax), for calculating the likelihood of the "no model" (NULL to skip)

# For calculating the evidence (sampling the prior)
calcEvidence <- FALSE
Nevsamp   <- 2e6  # Number of prior samples (=likelihood evaluations) to use in calculating evidence
Nparallel <- 1e4  # Number of samples to group into one parallel foreach loop
Ncores    <- 0    # Number of cores to use in parallel processing. Use all available if set to zero
verboseE  <- 1e4  # How often the likelihood evaluations should be printed to screen
priorSeed <- 100  # for sampling from the prior

# For sampling the posterior. Set just one of Nchain or Nwalker to be non-zero, the other to zero (to not use)
Npart <- 0 # if >1 then perform Npart-fold CV. If >nrow(obsdata) then it's set to this value. If 0 or 1, no CV is done.
samplePosterior <- TRUE # sample posterior using all the data (also calculates DIC)?
Nchain  <- 0  # if >1, the number of chain in parallel tempering. =1 is normal Metropolis sampling [=0 to use neither]
swapInt <- 10 # iteration interval at which a test for chain swaping is made
Nwalker <- 100 # the number of of walkers in emcee [=0 to not use emcee]
# Set initial values for all those parameters in thetaFlag which are TRUE.
# (Initial values assigned to fixed parameters - as defined by thetaXFlag - will be ignored.)
# If nothing is adjustable, then the thetaXInit is never used, so we can omit it entirely (or set to NULL).
theta1Init <- list(slope=0.5, tmid=70, offset=-20)
theta2Init <- list(sd=1)
theta3Init <- NULL
# Define sampling PDF for MCMC, or the initial distribution for the walkers in emcee.
# Give the SD for the transformed parameters, as applied by trans.theta().
# Parameters with a Gaussian prior are not transformed
# Parameters with a Gamma prior are logged, so SD is a scale factor
# slope is angle in radians
sampleSD   <- list(slope=0.01, tmid=1, offset=1, sd=0.05)
sampleCor  <- 0.0 # assume common correlation between all parameters
NburnIts   <- 0
Npostsamp  <- 200 # no. of posterior samples (per chain if using PT, per walker if using emcee)
verboseP   <- 10 # How often the MCMC posterior evaluations should be printed to screen
posteriorSeed <- 100 # for sampling from the posterior
